from csvtodb.Database import Database
from csvtodb.MysqlTable import MysqlTable
from csvtodb.Csv import Csv


class MysqlDb(Database, MysqlTable):

    def __repr__(self):
        return 'class to build new db'

    @classmethod
    def _build_db(cls, db_name: str, files: dict, engine: str):
        """
        build new db with table

        **files:**\n
        files need to be\n
        {\n
        'path_to_files_1': ((csv_filename_1, delimiter, quotechar), (csv_filename_1, delimiter, quotechar), etc...)\n
        'path_to_files_2': ((csv_filename_1, delimiter, quotechar), (csv_filename_1, delimiter, quotechar), etc...)\n
        }\n

        :return:
        """
        csv = Csv(filename='', filepath='', delimiter='', quoter='')
        file = f'CREATE DATABASE IF NOT EXISTS {db_name};\n\nUSE {db_name};\n\n'
        for i in files:
            for j in range(0, len(files[i])):
                csv.p_filepath = i
                csv.p_filename = files[i][j][0]
                csv.p_file_delimiter = files[i][j][1]
                csv.p_file_quoter = files[i][j][2]
                file += cls._build_table(csv=csv, engine=engine, temporary=False)
        return file

    @classmethod
    def _build_tables(cls, files: dict, engine: str):
        """
        build multiple table

        **files:**\n
        files need to be\n
        {\n
        'path_to_files_1': ((csv_filename_1, delimiter, quotechar), (csv_filename_1, delimiter, quotechar), etc...)\n
        'path_to_files_2': ((csv_filename_1, delimiter, quotechar), (csv_filename_1, delimiter, quotechar), etc...)\n
        }\n

        :return:
        """
        csv = Csv(filename='', filepath='', delimiter='', quoter='')
        file = ''
        for i in files:
            for j in range(0, len(files[i])):
                csv.p_filepath = i
                csv.p_filename = files[i][j][0]
                csv.p_file_delimiter = files[i][j][1]
                csv.p_file_quoter = files[i][j][2]
                file += cls._build_table(csv=csv, engine=engine, temporary=False)
        return file



import abc


class Database:

    def __repr__(self):
        return 'interface to create new database'

    @abc.abstractmethod
    def _build_db(self, db_name: str, files: dict, engine: str) -> str:
        """
        create new database
        :return:
        """
        pass



import re
from csvtodb.Csv import Csv
from csvtodb.Table import Table
from csvtodb.LaravelColumn import LaravelColumn


class LaravelTable(Table, LaravelColumn):
    def __repr__(self):
        return 'build table for laravel'

    @classmethod
    def _build_table(cls, csv: Csv,  engine: str, temporary: bool) -> str:
        """
        build new migration table for Laravel
        for the up function
        :param csv:
        :param engine:
        :param temporary:
        :return:
        """
        columns: dict = cls._define_col_type(csv)

        new_table: str = '\n\t\tSchema::create("%s", function(Blueprint $table){\n' % csv.p_filename
        new_table += f'\t\t\t$table->engine = "{engine}";\n'
        if temporary:
            new_table += '\t\t\t$table->temporary();\n'
        new_table += cls._primary()

        for column in columns:
            if columns[column]:
                if column == 'integer':
                    for i in columns[column]:
                        content: list = csv.column_content(column=i + 1)
                        name: str = content.pop(0)
                        new_table += cls._integer(column_value=content, column_name=name)
                elif column == 'decimal':
                    for i in columns[column]:
                        content: list = csv.column_content(column=i + 1)
                        name: str = content.pop(0)
                        new_table += cls._decimal(column_value=content, column_name=name)
                elif column == 'string':
                    for i in columns[column]:
                        content: list = csv.column_content(column=i + 1)
                        name: str = content.pop(0)
                        new_table += cls._string(column_value=content, column_name=name)
                elif column == 'date':
                    for i in columns[column]:
                        content: list = csv.column_content(column=i + 1)
                        name: str = content.pop(0)
                        new_table += cls._date(column_value=content, column_name=name)
                elif column == 'foreign':
                    for i in columns[column]:
                        content: list = csv.column_content(column=i+1)
                        name: str = content.pop(0)
                        new_table += f'{cls._foreign(column_value=content, column_name=name)},\n'

        new_table += '\t\t});\n'
        return new_table

    @classmethod
    def _down(cls, csv: Csv):
        """
        used for the down function to remove table

        :param csv:
        :return:
        """
        return f'\t\tSchema::dropIfExists("{csv.p_filename}");\n'

    @classmethod
    def _define_col_type(cls, csv: Csv):
        """
        define column type

        :param csv:
        :return:
        """
        date_pattern: str = r'^([0-9]{2}|[0-9]{1})([-|\/|.][0-9]{2}|[-|\/|.][0-9]{1})[-|\/|.][0-9]{4}\s[0-9]{2}\:[0-9]{2}\:[0-9]{2}$|' \
                            r'^([0-9]{2}|[0-9]{1})[-|\/|.]([0-9]{2}|[0-9]{1})[-|\/|.][0-9]{4}$|^[0-9]{4}$|' \
                            r'^[0-9]{2}\:[0-9]{2}\:[0-9]{2}$'

        content: list = csv.row_list(limit=1)[0]
        col_name: tuple = csv.column_name()

        col_type: dict = {
            'integer': [],
            'decimal': [],
            'string': [],
            'date': [],
            'foreign': []
        }

        # check which col is str or numeric
        for i in range(0, len(col_name)):
            if re.search(r'_id$', col_name[i]):
                col_type['foreign'].append(i)

        for i in range(0, len(content)):
            try:
                int(content[i])
                if i not in col_type['foreign']:
                    col_type['integer'].append(i)
            except ValueError:
                try:
                    float(content[i])
                    col_type['decimal'].append(i)
                except ValueError:
                    col_type['date' if re.match(date_pattern, content[i], re.MULTILINE) else 'string'].append(i)

        return col_type



import abc


class Column(abc.ABC):

    def __repr__(self):
        return 'interface to make sql column'

    @abc.abstractmethod
    def _integer(self, column_value: list, column_name: str) -> str:
        """
        create column for int value\n

        **value:**\n
        dict of all value in the selected column\n

        **column_name**\n
        name given to the column\n

        :param column_value: dict
        :param column_name: str
        :return: str
        """
        pass

    @abc.abstractmethod
    def _decimal(self, column_value: list, column_name: str) -> str:
        """
        create column for float value\n

        **value:**\n
        dict of all value in the selected column\n

        **column_name**\n
        name given to the column\n

        :param column_value: dict
        :param column_name: str
        :return: str
        """
        pass

    @abc.abstractmethod
    def _string(self, column_value: list, column_name: str, char_set: str, collation: str) -> str:
        """
        create column for string value\n

        **value:**\n
        dict of all value in the selected column\n

        **column_name**\n
        name given to the column\n

        **char_set**\n
        Character set to use\n

        **collation**\n
        collation to use\n

        :param column_value: dict
        :param column_name: str
        :param char_set: str
        :param collation: str
        :return: str
        """
        pass

    @abc.abstractmethod
    def _date(self, column_value: list, column_name: str) -> str:
        """
        create column for date value\n

        **value:**\n
        dict of all value in the selected column\n

        **column_name**\n
        name given to the column\n

        :param column_value: dict
        :param column_name: str
        :return: str
        """
        pass

    @abc.abstractmethod
    def _primary(self) -> str:
        """
        create the primary key
        :return: str
        """
        pass

    @abc.abstractmethod
    def _foreign(self, column_value: list, column_name: str) -> str:
        pass



import csv
import json
import re


class Csv:

    def __init__(self, filename: str, filepath: str, delimiter: str = ',', quoter: str = '"'):
        self.__filename: str = filename
        self.__filepath: str = filepath
        self.__delimiter: str = delimiter
        self.__quoter: str = quoter

    def __repr__(self):
        return 'class to edit or read csv data'

    def total_column(self) -> int:
        """
        return total column in the csv
        :return: int
        """
        return len(tuple(csv.reader(open(f'{self.__filepath}/{self.__filename}', 'r'),
                                    delimiter=self.__delimiter,
                                    quotechar=self.__quoter))[0])

    def row_list(self, limit: int = 0) -> tuple:
        """
        get list of row:\n
        limit = 1 return the first row\n
        limit = 2 return the first two rows\n
        etc...\n
        and start at the first data row

        :return:
        """
        if limit > 0:
            return tuple(csv.reader(open(f'{self.__filepath}/{self.__filename}.csv', 'r'),
                                    delimiter=self.__delimiter,
                                    quotechar=self.__quoter))[1:limit + 1]
        else:
            return tuple(csv.reader(open(f'{self.__filepath}/{self.__filename}.csv', 'r'),
                                    delimiter=self.__delimiter,
                                    quotechar=self.__quoter))[1:]

    def column_content(self, column: int):
        """
        return all row for one column\n

        **column -> int:**\n
        this param must contain column index from your file
        column index start at 1 not 0\n

        :param column: str
        :return: dict
        """
        column -= 1
        data: list = []
        with open(f'{self.__filepath}/{self.__filename}.csv', 'r') as file:
            read = csv.reader(file, delimiter=self.__delimiter, quotechar=self.__quoter)
            for row in read:
                data.append(row[column])
        file.close()
        return data

    def column_name(self) -> tuple:
        """
        return name of column
        :return:
        """
        column: tuple = tuple(csv.reader(open(f'{self.__filepath}/{self.__filename}.csv', 'r'),
                                         delimiter=self.__delimiter, quotechar=self.__quoter))[0]
        return tuple(column)

    def total_row(self) -> int:
        """
        return total of row

        :return:
        """
        return len(tuple(csv.reader(open(f'{self.__filepath}/{self.__filename}.csv', 'r'),
                                    delimiter=self.__delimiter, quotechar=self.__quoter)))

    def update_column_name(self, old, new):
        """
        set the name of one or more column\n

        **old:**\n
        column name to change in the csv_file, if you want change
        only one name it must be str and if you want to change name of multiple column
        it must be tuple\n

        **new:**\n
        new is the new column name, if you have precise str in actual this param must be str else
        this param must be tuple\n

        :return:
        """
        # check if param are correct
        if (type(old) is not str and type(old) is not tuple) or (type(new) is not str and type(new) is not tuple):
            raise TypeError('actual and new must be str or tuple')

        # check if param match
        if type(old) is str and type(new) is not str:
            raise TypeError('new must be str')
        elif type(old) is tuple:
            if type(new) is not tuple:
                raise TypeError('new must be tuple')
            # check if same number elem in each tuple
            if len(old) == len(new):
                for value in old:
                    if type(value) is not str:
                        raise TypeError('tuple for actual must contain only str')
                for value in new:
                    if type(value) is not str:
                        raise TypeError('tuple for new must contain only str')
            else:
                raise ValueError('You must have the same element in both tuples')

        try:
            # get value
            csv_data = list(csv.reader(open(f'{self.__filepath}/{self.__filename}', 'r'),
                                       delimiter=self.__delimiter, quotechar=self.__quoter))

            # change value
            if isinstance(old, str):
                for name in csv_data[0]:
                    if name == old:
                        i: int = csv_data[0].index(name)
                        csv_data[0][i] = new
                        del i
            else:
                for name in csv_data[0]:
                    if name in old:
                        csv_data[0][csv_data[0].index(name)] = new[old.index(name)]

            # write value
            csv.writer(open(f'{self.__filepath}/{self.__filename}', 'w', newline='')).writerows(csv_data)
            del csv_data, name
        except csv.Error as e:
            print(f'Error with the csv_file: {e}')

    def new_row(self, data: list, start: bool = False):
        """
        insert new row in the csv_file\n

        **data:**\n
        data represent the data you want to insert in the row

        **start:**\n
        default is False, set it to True if you want insert row at the start of file
        (after the first row which define column)\n

        :return:
        """
        try:
            csv_data: list = list(csv.reader(open(f'{self.__filepath}/{self.__filename}', 'r'), delimiter=self.__delimiter, quotechar=self.__quoter))
            if len(data) == len(csv_data[0]):
                csv_data.append(data) if not start else csv_data.insert(1, data)
            else:
                raise ValueError(f'your data list must have {len(csv_data[0])} column')
            csv.writer(open(f'{self.__filepath}/{self.__filename}', 'w', newline='')).writerows(csv_data)
        except csv.Error as e:
            print(e)

    def update_row(self, old, new, row: int = 0):
        """
        change value in row for one or more data

        **old:**\n
        the actual value in the row\n

        **new:**\n
        the new value to set in the row\n

        **row:**\n
        the line where to do the changes if row = 0 the modification are done on all row\n

        :return:
        """
        # check type
        if not isinstance(old, str) and not isinstance(old, tuple):
            raise TypeError('old must be str or tuple')
        if not isinstance(new, str) and not isinstance(new, tuple):
            raise TypeError('new must be str or tuple')
        if type(old) != type(new):
            raise TypeError('old and new must have same type')
        if isinstance(old, tuple):
            if len(old) != len(new):
                raise ValueError('old and new must have same length')
            for elem in old:
                if not isinstance(elem, str) and not isinstance(elem, int):
                    raise ValueError('old must contain str or int value only')
            for elem in new:
                if not isinstance(elem, str) and not isinstance(elem, int):
                    raise ValueError('new must contain str or int value only')

        try:
            csv_data = list(csv.reader(open(f'{self.__filepath}/{self.__filename}', 'r'), delimiter=self.__delimiter, quotechar=self.__quoter))

            if row == 0:
                if isinstance(old, str):
                    for elem in csv_data:
                        if old in elem:
                            elem[elem.index(old)] = new
                else:
                    for elem in csv_data:
                        for value in old:
                            if value in elem:
                                elem[elem.index(value)] = new[old.index(value)]
            else:
                if isinstance(old, str) and old in csv_data[row]:
                    csv_data[row][csv_data[row].index(old)] = new
                else:
                    for value in old:
                        if value in csv_data[row]:
                            csv_data[row][csv_data[row].index(value)] = new[old.index(value)]

            csv.writer(open(f'{self.__filepath}/{self.__filename}', 'w', newline='')).writerows(csv_data)
        except csv.Error as e:
            print(e)

    def to_json(self, path: str, filename: str, get_col_name: bool = False, key_on_value: bool = False) -> bool:
        """
        create json file with data from csv_file, return True if the file already exist return False\n

        **_filename:**\n
        _filename represent the name to use when file is create\n

        **path:**\n
        path is where to save the file\n

        **get_col_name:**\n
        add column name in the json\n

        **key_on_value:**\n
        This will refer each value in each row with for key the corresponding column\n

        :param path: str
        :param filename: str
        :param get_col_name: bool
        :param key_on_value: bool
        :return: bool
        """
        try:
            with open(f'{self.__filepath}/{self.__filename}', 'r') as file:
                csv_data: list = list(csv.reader(file, delimiter=self.__delimiter, quotechar=self.__quoter))
                total: int = 1
                # check if need col name
                if get_col_name:
                    data: dict = {'column': csv_data[0]}
                else:
                    data: dict = {}

                # check if need key on value
                if key_on_value:
                    column: list = csv_data[0]
                    del csv_data[0]
                    for elem in csv_data:
                        row: dict = {}
                        for value in elem:
                            row[column[elem.index(value)]] = value
                        data[total] = row
                        total += 1
                else:
                    del csv_data[0]
                    for elem in csv_data:
                        data[total] = elem
                        total += 1
            file.close()

            with open(f'{path}/{filename}.json', 'x') as file:
                json.dump(data, file)
            file.close()

            return True
        except csv.Error as e:
            print(f'error with your csv_file {e}')
        except FileExistsError:
            return False

    def format_column_name(self):
        """
        format column name
        :return:
        """
        data: list = list(csv.reader(open(f'{self.__filepath}/{self.__filename}.csv', 'r'),
                                     delimiter=self.__delimiter, quotechar=self.__quoter))
        data[0] = [re.sub(r' ', '_', data[0][i], 0, re.MULTILINE) for i in range(0, len(data[0]))]
        csv.writer(open(f'{self.__filepath}/{self.__filename}.csv', 'w', newline='')).writerows(data)

    def set_filepath(self, filepath: str):
        """
        set file to use
        :param filepath:
        """
        self.__filepath = filepath

    def get_filepath(self) -> str:
        """
        return path or only _filename used
        :return: str
        """
        return self.__filepath

    def set_filename(self, filename: str):
        """
        set _filename to use
        :param filename:
        """
        self.__filename = filename

    def get_filename(self) -> str:
        """
        return _filename used
        :return: str
        """
        return self.__filename

    def set_delimiter(self, delimiter: str):
        """
        set delimiter in the file
        :param delimiter:
        """
        self.__delimiter = delimiter

    def get_delimiter(self) -> str:
        """
        return delimiter used
        :return: str
        """
        return self.__delimiter

    def set_quoter(self, quoter: str):
        """
        set delimiter in the file
        :param quoter:
        """
        self.__quoter = quoter

    def get_quoter(self) -> str:
        """
        return delimiter used
        :return: str
        """
        return self.__quoter

    p_filepath = property(fget=get_filepath, fset=set_filepath)
    p_filename = property(fget=get_filename, fset=set_filename)
    p_file_delimiter = property(fget=get_delimiter, fset=set_delimiter)
    p_file_quoter = property(fget=get_quoter, fset=set_quoter)



import re

from csvtodb.Csv import Csv
from csvtodb.MysqlDb import MysqlDb


class Mysql(MysqlDb):

    def __repr__(self):
        return 'build db or table or both with using mysql'

    @classmethod
    def new_db(cls, save_in: str, db_name: str, files: dict, engine: str = 'INNODB'):
        """
        build new db with table\n

        **files:**\n
        files need to be\n
        {\n
        'path_to_files_1': ((csv_filename_1, delimiter, quotechar), (csv_filename_1, delimiter, quotechar), etc...)\n
        'path_to_files_2': ((csv_filename_1, delimiter, quotechar), (csv_filename_1, delimiter, quotechar), etc...)\n
        }\n

        :return:
        """
        try:
            with open(f'{save_in}/{db_name}.sql', 'x') as file:
                file.write(cls._build_db(db_name=db_name, files=files, engine=engine))
            file.close()
            return True
        except FileExistsError:
            return False

    @classmethod
    def new_table(cls, csv: Csv, filepath: str, filename: str, engine: str = 'INNODB',
                  temporary: bool = False) -> bool:
        """
        create new table in file\n

        **filepath:**\n
        where to save the file\n

        **_filename:**\n
        name of the saved file\n

        **return type:**\n
        if all is ok (file created and content added) return true
        else if there is any error return false

        :return: bool
        """
        try:
            with open(f'{filepath}/{filename}.sql', 'x') as file:
                file.write(cls._build_table(csv=csv, engine=engine, temporary=temporary))
            file.close()
            return True
        except FileExistsError:
            with open(f'{filepath}/{filename}.sql', 'a') as file:
                file.write(cls._build_table(csv=csv, engine=engine, temporary=temporary))
            file.close()
            return True

    @classmethod
    def new_tables(cls, filepath: str, filename: str, files: dict, engine: str = 'INNODB'):
        """
        build multiple table from multiple csv file\n

        **filepath:**\n
        where to save the file\n

        **_filename:**\n
        name of the saved file\n

        **files**\n
        content all files to build in table, it must be construct like this:\n
        files need to be\n
        {\n
        'path_to_files_1': ((csv_filename_1, delimiter, quotechar), (csv_filename_1, delimiter, quotechar), etc...)\n
        'path_to_files_2': ((csv_filename_1, delimiter, quotechar), (csv_filename_1, delimiter, quotechar), etc...)\n
        }\n

        :return:
        """
        try:
            with open(f'{filepath}/{filename}.sql', 'x') as file:
                file.write(cls._build_tables(files=files, engine=engine))
            file.close()
        except FileExistsError:
            with open(f'{filepath}/{filename}.sql', 'a') as file:
                file.write(cls._build_tables(files=files, engine=engine))
            file.close()

    @classmethod
    def new_seeder(cls, csv: Csv, filepath: str, filename: str):
        """
        build seeder from file

        **filepath:**\n
        where to save the file\n

        **filename:**\n
        name of the saved file\n

        :param csv:
        :param filepath:
        :param filename:
        :return:
        """
        column_name: tuple = csv.column_name()
        seeder = re.sub(r"'", '', f'INSERT INTO {csv.p_filename} {column_name} VALUES\n', 0, re.MULTILINE)
        rows = csv.row_list()
        for i, row in enumerate(rows):
            seeder += f'\t{tuple(row)},\n' if i != len(rows)-1 else f'\t{tuple(row)};'
        try:
            with open(f'{filepath}/{filename}.sql', 'x') as file:
                file.write(seeder)
            file.close()
        except FileExistsError:
            with open(f'{filepath}/{filename}.sql', 'a') as file:
                file.write(seeder)
            file.close()



import re
from csvtodb.Column import Column


class MysqlColumn(Column):
    __FOREIGN_REFERENCES: tuple = ('RESTRICT', 'CASCADE', 'NO ACTION', 'SET DEFAULT', 'SET NULL')  # on_update/on_delete
    __SIGNED: dict = {
        'tinyint': (-128, 127),
        'smallint': (-32768, 32767),
        'mediumint': (-8388608, 8388607),
        'int': (-2147483648, 2147483647),
        'bigint': (-2 ** 63, 2 ** 63 - 1),
    }  # signed type name: value
    __UNSIGNED: dict = {
        'tinyint': (0, 255),
        'smallint': (0, 65535),
        'mediumint': (0, 16777215),
        'int': (0, 4294967295),
        'bigint': (0, 2 ** 64 - 1),
    }  # unsigned type name: value
    __STRING: dict = {
        'char': 255,
        'varchar': 65.535,
        'tinytext': 2 ** 8,
        'text': 2 ** 16,
        'mediumtext': 2 ** 24,
        'longtext': 2 ** 32,
    }  # string type name: value
    __DATE: tuple = ('DATE', 'DATETIME', 'TIME', 'YEAR', 'TIMESTAMP')  # date type name
    __DECIMAL: tuple = ('FLOAT', 'DOUBLE')

    def __repr__(self):
        return 'create new column for mysql'

    @classmethod
    def _integer(cls, column_value: list, column_name: str) -> str:
        column: str = column_name
        min_val = int(min(column_value))
        max_val = int(max(column_value))

        # select type
        if min_val < 0:
            for i in cls.__SIGNED:
                if (cls.__SIGNED[i][0]) <= max_val <= (cls.__SIGNED[i][1]):
                    column += f' {i.upper()} __SIGNED'
                    break
        else:
            for i in cls.__UNSIGNED:
                if (cls.__UNSIGNED[i][0]) <= max_val <= (cls.__UNSIGNED[i][1]):
                    column += f' {i.upper()} UNSIGNED'
                    break

        # check if need to add extra keywords
        if len(str(min_val)) == len(str(max_val)):
            column += ' ZEROFILL'
        else:
            column += ' NULL' if len(str(min_val)) == 0 else ' NOT NULL'

        return column

    @classmethod
    def _decimal(cls, column_value: list, column_name: str) -> str:
        column: str = column_name

        range_value = (min(column_value), max(column_value))
        range_value_str: tuple = (len(range_value[0]), len(range_value[1]))
        integer, decimal = range_value[0 if range_value_str[0] > range_value_str[1] else 1].split('.')

        zerofill: bool = True if range_value_str[0] == range_value_str[1] else False

        column += f' {cls.__DECIMAL[1]}' if len(decimal) > 2 else f' {cls.__DECIMAL[0]}'
        column += ' __SIGNED' if int(integer) < 0.0 else ' UNSIGNED'

        if zerofill:
            column += ' ZEROFILL'

        column += ' NOT NULL' if not range_value_str[0] == 0 else ' NULL'
        return column

    @classmethod
    def _string(cls, column_value: list, column_name: str, charset: str = 'utf8',
                collation: str = 'utf8_general_ci') -> str:
        column: str = column_name
        min_val = len(min(column_value, key=len))
        max_val = len(max(column_value, key=len))
        is_text: bool = False

        # for i in column_value:
        #     if re.match(r'[\n\t]', column_value[i], re.MULTILINE):
        #         is_text = True
        #         break

        # set the type
        if is_text:
            for i in cls.__STRING:
                if i != 'char' and i != 'varchar':
                    if max_val <= cls.__STRING[i]:
                        column += f' {i.upper()}({max_val})'
                        break
        else:
            column += f' {"CHAR" if min_val == max_val and max_val <= 255 else "VARCHAR"}({max_val})'
            column += f' CHARACTER SET {charset} COLLATE {collation}'

        # check if null
        column += ' NULL' if min_val == 0 else ' NOT NULL'

        return column

    @classmethod
    def _date(cls, column_value: list, column_name: str) -> str:
        column: str = column_name

        if re.match(r'^([0-9]{2}|[0-9])[-/.]([0-9]{2}|[0-9])[-/.][0-9]{4}$|^[0-9]{4}$', column_value[0], re.MULTILINE):
            column += f' {cls.__DATE[0]} NOT NULL'
        elif re.match(r'^[0-9]{4}$', column_value[0], re.MULTILINE):
            column += f' {cls.__DATE[3]} NOT NULL'
        elif re.match(r'^[0-9]{2}:[0-9]{2}:[0-9]{2}$', column_value[0], re.MULTILINE):
            column += f' {cls.__DATE[2]} NOT NULL'
        elif re.match(r'^([0-9]{2}|[0-9])([-/.][0-9]{2}|[-/.][0-9])[-/.][0-9]{4}\s[0-9]{2}:[0-9]{2}:[0-9]{2}$',
                      column_value[0], re.MULTILINE):
            timestamp: bool = True
            for i in column_value:
                val, hour = column_value[i].split(' ')
                day, month, year = val.split('/')
                hour, minute, second = hour.split(':')
                if 1970 <= int(year) <= 2038:
                    if ((int(year) == 2038 and int(day) >= 19) or (int(year) == 1970 and int(day) > 1)) and \
                            (int(month) == 1 and int(hour) == 0 and int(minute) == 0 and int(second) > 1):
                        timestamp = False
                        break
            column += f' {cls.__DATE[4 if timestamp else 1]} NOT NULL'
        return column

    @classmethod
    def _primary(cls) -> str:
        return f'\nid INT UNSIGNED NOT NULL UNIQUE AUTO_INCREMENT PRIMARY KEY'

    @classmethod
    def _foreign(cls, column_value: list, column_name: str) -> str:
        """
        build foreign key for mysql
        :return:
        """
        table, field = column_name.split('_')
        column = column_name

        for i in cls.__UNSIGNED:
            if i == 'int' or i == 'bigint' and (cls.__UNSIGNED[i][0]) <= int(max(column_value)) <= (cls.__UNSIGNED[i][1]):
                column += f' {i.upper()} UNSIGNED NOT NULL,\n'
                break

        column += f'INDEX ix_{table}({column_name}),\n'
        column += f'FOREIGN KEY ({column_name})' \
                  f'\n\tREFERENCES {table}({field})' \
                  f'\n\tON DELETE {cls.__FOREIGN_REFERENCES[1]}' \
                  f'\n\tON UPDATE {cls.__FOREIGN_REFERENCES[1]}'

        return column



import abc
from csvtodb.Csv import Csv


class Table:
    def __repr__(self):
        return 'interface to make sql table'

    @abc.abstractmethod
    def _build_table(self, csv: Csv, engine: str, temporary: bool) -> str:
        """
        create new sql table

        :param csv: Csv
        :param engine: str
        :param temporary: bool
        :return: str
        """
        pass

    @abc.abstractmethod
    def _define_col_type(self, csv: Csv):
        pass



import re
from csvtodb.Table import Table
from csvtodb.MysqlColumn import MysqlColumn
from csvtodb.Csv import Csv


class MysqlTable(Table, MysqlColumn):
    __ENGINE: tuple = ('innodb', 'myisam', 'memory', 'csv')

    def __repr__(self):
        return 'class to build sql table for mysql'

    @classmethod
    def _build_table(cls, csv: Csv, engine: str, temporary: bool) -> str:
        res: dict = {}
        columns: dict = cls._define_col_type(csv)
        if engine.lower() in cls.__ENGINE:
            for column in columns:
                if columns[column]:
                    if column == 'integer':
                        for i in columns[column]:
                            content: list = csv.column_content(column=i+1)
                            name: str = content.pop(0)
                            res[i] = f'{cls._integer(column_value=content, column_name=name)},\n'
                    elif column == 'decimal':
                        for i in columns[column]:
                            content: list = csv.column_content(column=i+1)
                            name: str = content.pop(0)
                            res[i] = f'{cls._decimal(column_value=content, column_name=name)},\n'
                    elif column == 'string':
                        for i in columns[column]:
                            content: list = csv.column_content(column=i+1)
                            name: str = content.pop(0)
                            res[i] = f'{cls._string(column_value=content, column_name=name)},\n'
                    elif column == 'date':
                        for i in columns[column]:
                            content: list = csv.column_content(column=i+1)
                            name: str = content.pop(0)
                            res[i] = f'{cls._date(column_value=content, column_name=name)},\n'
                    elif column == 'foreign':
                        for i in columns[column]:
                            content: list = csv.column_content(column=i+1)
                            name: str = content.pop(0)
                            res[i] = f'{cls._foreign(column_value=content, column_name=name)},\n'

            new_table: str = f'CREATE {"TEMPORARY" if temporary else ""} TABLE IF NOT EXISTS {csv.p_filename}('
            new_table += f'\t{cls._primary()},\n'
            res[max(res)] = res[max(res)][0:-2]+'\n'
            for i in range(0, len(res)):
                new_table += res[i]
            new_table += f')ENGINE="{engine.upper()}";\n\n'
            return new_table
        else:
            raise ValueError(f'engine not supported')

    @classmethod
    def _define_col_type(cls, csv: Csv) -> dict:
        """
        define column type

        :param csv:
        :return:
        """
        date_pattern: str = r'^([0-9]{2}|[0-9]{1})([-|\/|.][0-9]{2}|[-|\/|.][0-9]{1})[-|\/|.][0-9]{4}\s[0-9]{2}\:[0-9]{2}\:[0-9]{2}$|' \
                            r'^([0-9]{2}|[0-9]{1})[-|\/|.]([0-9]{2}|[0-9]{1})[-|\/|.][0-9]{4}$|^[0-9]{4}$|' \
                            r'^[0-9]{2}\:[0-9]{2}\:[0-9]{2}$'

        content: list = csv.row_list(limit=1)[0]
        col_name: tuple = csv.column_name()
        col_type: dict = {
            'integer': [],
            'decimal': [],
            'string': [],
            'date': [],
            'foreign': []
        }

        # check which col is str or numeric
        for i in range(0, len(col_name)):
            if re.search(r'_id$', col_name[i]):
                col_type['foreign'].append(i)

        for i in range(0, len(content)):
            try:
                int(content[i])
                if i not in col_type['foreign']:
                    col_type['integer'].append(i)
            except ValueError:
                try:
                    float(content[i])
                    col_type['decimal'].append(i)
                except ValueError:
                    col_type['date' if re.match(date_pattern, content[i], re.MULTILINE) else 'string'].append(i)

        return col_type



from csvtodb.Csv import Csv
from csvtodb.LaravelTable import LaravelTable


class Laravel(LaravelTable):
    def __repr__(self):
        return 'build seeder and table migration files for laravel from csv'

    @classmethod
    def new_migration(cls, csv: Csv, filepath: str, filename: str, engine: str = 'InnoDB',
                      temporary: bool = False) -> bool:
        """
        build new migration file for laravel

        order step :\n
        - create your migration file with your laravel
        - set new csv with the csv class
        - call this method
        - run program

         your migration is now build congrats !\n

         **csv:**\n
         the csv you initialize before\n

         **filepath:**\n
         the path to the migration folder in your Laravel (absolute path recommended to avoid errors)\n

         **filename:**\n
         the name of your migration file create in your Laravel\n

         **engine:**\n
         engine you want to use for your table\n

         **temporary:**\n
         if your table need to be temporary set it to True\n

        :param csv: Csv
        :param filepath: str
        :param filename: str
        :param engine: str
        :param temporary: bool
        :return: bool
        """
        migration = '<?php\n\n' \
                    'use Illuminate\\Database\\Migrations\\Migration;\n' \
                    'use Illuminate\\Database\\Schema\\Blueprint;\n' \
                    'use Illuminate\\Support\\Facades\\Schema;\n\n' \
                    f'class {csv.p_filename} extends Migration\n' \
                    '{\n' \
                    '\tpublic function up()\n\t{'
        migration += cls._build_table(csv=csv, engine=engine, temporary=temporary)
        migration += '\t}\n\n'
        migration += '\tpublic function down()\n\t{\n'
        migration += cls._down(csv=csv)
        migration += '\t}\n'
        migration += '}\n'

        try:
            with open(f'{filepath}/{filename}.php', 'x') as file:
                file.write(migration)
            file.close()
        except FileExistsError:
            with open(f'{filepath}/{filename}.php', 'w') as file:
                file.write(migration)
            file.close()
        return True

    @classmethod
    def new_seeder(cls, csv: Csv, filepath: str, filename: str) -> bool:
        """
        build new seeder for laravel

        :param csv: Csv
        :param filepath: str
        :param filename: str
        :return: str
        """
        column_name = csv.column_name()
        content = csv.row_list()
        seeder = '<?php\n\n' \
                 'use Illuminate\\Database\\Seeder;\n' \
                 'use Illuminate\\Support\\Facades\\DB;\n\n' \
                 f'class {filename} extends Seeder\n' \
                 '{\n' \
                 '\tpublic function run()\n' \
                 '\t{\n' \
                 f'\t\tDB::table("{csv.p_filename}")->insert([\n' \

        for i in range(0, len(content)):
            seeder += '\t\t\t[\n'
            for j in range(0, len(content[i])):
                    seeder += f'\t\t\t\t"{column_name[j]}" => "{content[i][j]}",\n'
            seeder += '\t\t\t]\n' if i == len(content)-1 else '\t\t\t],\n'

        seeder += '\t\t]);\n' \
                  '\t}\n' \
                  '}'
        try:
            with open(f'{filepath}/{filename}.php', 'w') as file:
                file.write(seeder)
            file.close()
            return True
        except Exception as e:
            print(e)
            return False



import re
from csvtodb.Column import Column


class LaravelColumn(Column):
    __FOREIGN_REFERENCES: tuple = ('restrict', 'cascade', 'no action', 'set default', 'set null')  # on_update/on_delete
    __SIGNED: dict = {
        'tinyInteger': (-128, 127),
        'smallInteger': (-32768, 32767),
        'mediumInteger': (-8388608, 8388607),
        'integer': (-2147483648, 2147483647),
        'bigInteger': (-2 ** 63, 2 ** 63 - 1),
    }  # signed type name: value
    __UNSIGNED: dict = {
        'unsignedTinyInteger': (0, 255),
        'unsignedSmallInteger': (0, 65535),
        'unsignedMediumInteger': (0, 16777215),
        'unsignedInteger': (0, 4294967295),
        'unsignedBigInteger': (0, 2 ** 64 - 1),
    }  # unsigned type name: value
    __STRING: dict = {
        'char': 255,
        'string': 65.535
    }  # string type name: value
    __TEXT: dict = {
        'tinyText': 2 ** 8,
        'text': 2 ** 16,
        'mediumText': 2 ** 24,
        'longText': 2 ** 32
    }  # text type name: value
    __DATE: tuple = ('date', 'datetime', 'time', 'year', 'timestamp')  # date type name
    __DECIMAL: tuple = ('float', 'double')  # decimal type name

    def __repr__(self):
        return 'build column for laravel'

    @classmethod
    def _integer(cls, column_value: list, column_name: str) -> str:
        column: str = '\t\t\t$table->'
        min_val = int(min(column_value))
        max_val = int(max(column_value))

        # select type
        if min_val < 0:
            for i in cls.__SIGNED:
                if (cls.__SIGNED[i][0]) <= max_val <= (cls.__SIGNED[i][1]):
                    column += f'{i}("{column_name}")'
                    break
        else:
            for i in cls.__UNSIGNED:
                if (cls.__UNSIGNED[i][0]) <= max_val <= (cls.__UNSIGNED[i][1]):
                    column += f'{i}("{column_name}")'
                    break

        # check if need to add extra keywords
        if len(str(min_val)) == 0:
            column += '->nullable()'
        column += ';\n'
        return column

    @classmethod
    def _decimal(cls, column_value: list, column_name: str) -> str:
        column: str = '\t\t\t$table->'

        range_value = (min(column_value), max(column_value))
        range_value_str: tuple = (len(re.sub(r'\D+', '', range_value[0], 0, re.MULTILINE)),
                                  len(re.sub(r'\D+', '', range_value[1], 0, re.MULTILINE)))
        integer, decimal = range_value[range_value_str.index(max(range_value_str))].split('.')

        if int(integer) < 0.000:
            column += f'{cls.__DECIMAL[1 if len(decimal) > 2 else 0]}->("{column_name}")'
        else:
            total = len(integer) + len(decimal)
            column += f'unsignedDecimal("{column_name}", {total}, {len(decimal)})'

        if range_value_str[0] == 0:
            column += '->nullable()'
        column += ';\n'
        return column

    @classmethod
    def _string(cls, column_value: list, column_name: str, charset: str = 'utf8',
                collation: str = 'utf8_general_ci') -> str:
        column: str = '\t\t\t$table->'
        min_val = len(min(column_value, key=len))
        max_val = len(max(column_value, key=len))
        is_text: bool = False

        for i in column_value:
            if re.match(r'[\n\t]', column_value[i], re.MULTILINE):
                is_text = True
                break

        # set the type
        if is_text:
            for i in cls.__TEXT:
                if max_val <= cls.__TEXT[i]:
                    column += f' {i}({max_val})'
                    break
        else:
            column += f'{"char" if min_val == max_val and max_val <= cls.__STRING[0] else "string"}("{column_name}", {max_val})'

        # check if null
        if min_val == 0:
            column += '->nullable()'

        column += f'->charset("{charset}")->collation("{collation}")'
        column += ';\n'
        return column

    @classmethod
    def _date(cls, column_value: list, column_name: str) -> str:
        column: str = '\t\t\t$table->'

        if re.match(r'^([0-9]{2}|[0-9])[-/.]([0-9]{2}|[0-9])[-/.][0-9]{4}$|^[0-9]{4}$', column_value[0], re.MULTILINE):
            column += f'{cls.__DATE[0]}("{column_name}")'
        elif re.match(r'^[0-9]{4}$', column_value[0], re.MULTILINE):
            column += f'{cls.__DATE[3]}("{column_name}")'
        elif re.match(r'^[0-9]{2}:[0-9]{2}:[0-9]{2}$', column_value[0], re.MULTILINE):
            column += f'{cls.__DATE[2]}("{column_name}")'
        elif re.match(r'^([0-9]{2}|[0-9])([-/.][0-9]{2}|[-/.][0-9])[-/.][0-9]{4}\s[0-9]{2}:[0-9]{2}:[0-9]{2}$',
                      column_value[0], re.MULTILINE):
            timestamp: bool = True
            for i in column_value:
                val, hour = column_value[i].split(' ')
                day, month, year = val.split('/')
                hour, minute, second = hour.split(':')
                if 1970 <= int(year) <= 2038:
                    if ((int(year) == 2038 and int(day) >= 19) or (int(year) == 1970 and int(day) > 1)) and \
                            (int(month) == 1 and int(hour) == 0 and int(minute) == 0 and int(second) > 1):
                        timestamp = False
                        break
            column += f' {cls.__DATE[4 if timestamp else 1]}("{column_name}")'
            column += '->useCurrentOnUpdate()'
            column += '->useCurrent()'

        column += ';\n'
        return column

    @classmethod
    def _primary(cls) -> str:
        return f'\n\t\t\t$table->id();\n'

    @classmethod
    def _foreign(cls, column_value: list, column_name: str) -> str:
        """
        build foreign key for mysql
        :return:
        """
        table, field = column_name.split('_')
        column = f'\n\t\t\t$table->foreignId("{column_name}")' \
                 f'->constrained("{table}")' \
                 f'->onUpdate("{cls.__FOREIGN_REFERENCES[1]}")' \
                 f'->onDelete("{cls.__FOREIGN_REFERENCES[1]}");\n'

        return column



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="csvtodb",
    version="1.1.1",
    author="akk0ga",
    author_email="ttn.glock@orange.fr",
    description="Make it easier to build database/table/seeders for DBMS or framework like laravel",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/akk0ga/CsvToDB",
    classifiers=[
        "Programming Language :: Python :: 3.9",
        "License :: OSI Approved :: MIT License",
        "Intended Audience :: Developers",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ],
    packages=setuptools.find_packages(),
    keywords=['csv', 'db', 'database', 'sql', 'framework', 'mysql'],
    python_requires=">=3.6",
)
